package de.hft.stuttgart.citydoctor2.healer.gui;

import java.io.IOException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.gui.MainWindow;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TabPane;
import javafx.scene.control.TreeItem;
import javafx.scene.control.TreeView;
import javafx.scene.image.Image;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.Window;

public class ChooseFeatureDialog {

	private static final Logger logger = LogManager.getLogger(ChooseFeatureDialog.class);
	private Stage stage;
	private HealerController controller;

	@FXML
	private TreeView<SelectableFeature> buildingsView;

	@FXML
	private Button okButton;

	@FXML
	private Button cancelButton;
	
	@FXML
	private TabPane tabPane;

	public ChooseFeatureDialog(Window parent, HealerController controller) {
		FXMLLoader loader = new FXMLLoader(getClass().getResource("ChooseErrorFeatureDialog.fxml"));
		loader.setController(this);
		this.controller = controller;
		try {
			Parent dialogParent = loader.load();
			stage = new Stage();
			stage.getIcons()
					.add(new Image(MainWindow.class.getResourceAsStream("icons/CityDoctor-Logo-rot_klein.jpg")));
			stage.setScene(new Scene(dialogParent));
			stage.initOwner(parent);
			stage.initModality(Modality.APPLICATION_MODAL);
			stage.setTitle("Select Feature Geometry");
		} catch (IOException e) {
			logger.catching(e);
		}
	}

	public void initialize() {
		tabPane.getSelectionModel().selectedItemProperty().addListener((obs, oldV, newV) -> {
			Node content = oldV.getContent();
			if (content instanceof TreeView<?>) {
				TreeView<?> view = (TreeView<?>) content;
				view.getSelectionModel().clearSelection();
				okButton.setDisable(true);
			} else {
				throw new IllegalStateException("Programming error, tab does not contain TreeView as content");
			}
		});
		okButton.setDisable(true);
		okButton.setOnAction(ae -> {
			Node content = tabPane.getSelectionModel().getSelectedItem().getContent();
			if (content instanceof TreeView<?>) {
				TreeView<?> view = (TreeView<?>) content;
				Object selectedItem = view.getSelectionModel().getSelectedItem().getValue();
				if (selectedItem instanceof SelectableFeature) {
					controller.featureSelectionConfirmed((SelectableFeature) selectedItem);
					stage.close();
				} else {
					throw new IllegalStateException("Programming error, TreeView does not contain SelectableFeature");
				}
			} else {
				throw new IllegalStateException("Programming error, tab does not contain TreeView as content");
			}
		});
		cancelButton.setOnAction(ae -> stage.close());
		buildingsView.setRoot(new TreeItem<>(null));
		buildingsView.setShowRoot(false);
		buildingsView.getSelectionModel().setSelectionMode(SelectionMode.SINGLE);
		buildingsView.getSelectionModel().selectedItemProperty()
				.addListener((obs, oldV, newV) -> controller.itemSelected(newV, this));
		buildingsView.setCellFactory(param -> new ChooseDialogTreeCell());
	}

	public void show() {
		Platform.runLater(() -> stage.showAndWait());
	}

	public TreeView<SelectableFeature> getBuildingsTree() {
		return buildingsView;
	}

	public void setOkButtonEnabled(boolean enabled) {
		okButton.setDisable(!enabled);
	}

}
